unit uEditor;

interface

uses uDev;

function WinWidth : Word;
function WinHeight : Word;
function ChrDot(x:integer) : char;
procedure WindowText( pos:word; title:string );

type
  TAbsEditor = object
    m_pMem          : Pointer;
    m_pEdit         : Pointer;
    m_wMax          : word;
    m_ItemSize      : word;
    m_bc            : word;
    m_Address       : word;
    m_AddressSHL    : word;
    m_ItemWidth     : word;
    m_LineItems     : word;
    m_DumpT, m_DumpB, m_DumpL, m_DumpR, m_TextL, m_TextR : word;
    m_StartLine     : word;
    m_cx, m_cy      : word;

    function      MaxStartLine  : word;
    function      TotalLines    : word;
    procedure     XY( x,y:word );
    procedure     SetWord( w:Word );
    function      GetWord : Word;

    procedure     CalcAddress; virtual;
    procedure     PaintLine( y:word ); virtual;
    procedure     Paint; virtual;

    constructor   Init( pMem:Pointer; bc, itemSize, wMax, lineItems:word );
    destructor    Free; virtual;
    function      Execute : boolean; virtual;
  end;

implementation

uses TpWindow, TpCrt, TpString;



function WinWidth : Word;
begin
  with WindowP(CurrentWindow)^ do WinWidth := XH - XL;
end;

function WinHeight : Word;
begin
  with WindowP(CurrentWindow)^ do WinHeight := YH - YL;
end;

function ChrDot(x:integer) : char;
begin
  ChrDot := Chr(x);
end;

procedure WindowText( pos:word; title:string );
var
  x, y, l : word;
  color, color2 : byte;
begin
  x:=1;
  y:=0;
  l := Length(title);

  if Boolean(pos and 1) then x := WinWidth - 2 - l;
  if Boolean(pos and 2) then y := WinHeight + 2;

  color := WindowP(CurrentWindow)^.Draw.FAttr;
  color2 := color; {(color shl 4) or (color shr 4);}

  FastWriteWindow( '',   y, x+0,   color );
  FastWriteWindow( ' ',   y, x+1,   color2);
  FastWriteWindow( title, y, x+2,   color2);
  FastWriteWindow( ' ',   y, x+2+l, color2);
  FastWriteWindow( '',   y, x+3+l, color );
end;

constructor TAbsEditor.Init( pMem:Pointer; bc, itemSize, wMax, lineItems:word  );
begin
  m_pMem      := pMem;
  m_bc        := bc;
  m_itemSize  := itemSize;
  m_lineItems := lineItems;
  m_wMax      := wMax;
  m_pEdit     := nil;

  m_ItemWidth := m_itemSize*2+1;
  m_StartLine := 0;

  m_DumpT     := 1;           {full window}
  m_DumpB     := WinHeight+1; {full window}
  m_DumpL     := 8;           {after ' xxxx: ' }
  m_DumpR     := m_DumpL+m_LineItems*m_ItemWidth-2;
  m_TextL     := m_DumpR + 3;
  m_TextR     := m_TextL+m_LineItems-1;

  if TotalLines<m_DumpB then
    m_DumpB := TotalLines;

  XY(m_DumpL,m_DumpT);
end;

destructor TAbsEditor.Free;
begin
  if m_pEdit<>nil then
    FreeMem( m_pEdit, m_bc*m_itemSize );
end;

procedure TAbsEditor.SetWord( w:Word );
begin
  case m_ItemSize of
    1: PByteArray(m_pEdit)^[m_Address] := w;
    2: PWordArray(m_pEdit)^[m_Address] := w;
  end;
end;

function TAbsEditor.GetWord : Word;
begin
  case m_ItemSize of
    1: GetWord := PByteArray(m_pEdit)^[m_Address];
    2: GetWord := PWordArray(m_pEdit)^[m_Address];
  end;
end;

procedure TAbsEditor.XY( x,y:word );
begin
  m_cx := x;
  m_cy := y;
  GotoXY(m_cx,m_cy);
  CalcAddress;
end;


function TAbsEditor.MaxStartLine  : word;
begin
  MaxStartLine  := TotalLines - m_DumpB
end;

function TAbsEditor.TotalLines    : word;
begin
  TotalLines    := (m_bc-1) div m_LineItems+1;
end;

procedure TAbsEditor.CalcAddress;
begin
  m_Address := (m_StartLine+m_cy-1)*m_LineItems;
  m_AddressSHL := 0;
  if m_cx<=m_DumpR then
  begin
    m_Address := m_Address + ((m_cx-m_DumpL) div m_ItemWidth);
    m_AddressSHL := 4*(m_ItemWidth-2-((m_cx-m_DumpL) mod m_ItemWidth));
  end else
  begin
    m_Address := m_Address + (m_cx-m_TextL);
  end;
end;

procedure TAbsEditor.PaintLine( y:word );
var
  x : Word;
begin
  for x:=1 to WinWidth+1 do
  begin
    FastWriteWindow( ' ', y, x, $07 );
  end;
end;

procedure TAbsEditor.Paint;
var
  y : word;
begin
  for y:=m_DumpT to m_DumpB do
  begin
    PaintLine(y);
  end;
  CalcAddress;
end;

function TAbsEditor.Execute : boolean;
var
  key           : Word;
  bKey          : Integer;
  w             : Word;
label
  __down, __up, __right;
begin
  Execute := false;

  GetMem( m_pEdit, m_bc*m_itemSize );
  Move( m_pMem^, m_pEdit^, m_bc*m_itemSize );

  Paint;

  while true do
  begin
    key := ReadKeyWord;
    case key of

    $5300: {Del}
      begin
        case m_ItemSize of
        1: PByteArray(m_pEdit)^[m_Address] := PByteArray(m_pMem)^[m_Address];
        2: PWordArray(m_pEdit)^[m_Address] := PWordArray(m_pMem)^[m_Address];
        end;
        PaintLine( m_cy );
      end;

    $011B: {Esc}
      begin
        FreeMem( m_pEdit, m_bc*m_itemSize );
        m_pEdit := nil;
        Exit;
      end;

    $0F09: {Tab}
      if m_cx<=m_DumpR then
        XY(m_TextL+((m_cx-m_DumpL) div m_ItemWidth),m_cy)
      else
        XY(m_DumpL+m_ItemWidth*(m_cx-m_TextL),m_cy);

    $1C0D, {Enter}
    $1C0A: {C+Enter}
      begin
        Move( m_pEdit^, m_pMem^, m_bc*m_itemSize );
        FreeMem( m_pEdit, m_bc*m_itemSize );
        m_pEdit := nil;
        Exit;
      end;

    $4800: {Up}
__up:
      if m_cy<=1 then
      begin
        if m_StartLine<>0 then
        begin
          XY(m_cx,m_cy);
          Dec(m_StartLine);
          Paint;
        end;
      end else
      begin
        Dec(m_cy);
        XY(m_cx,m_cy);
      end;

    $5000: {Down}
__down:
      if m_Address<m_bc-m_LineItems then
      begin
        if m_cy>=m_DumpB then
        begin
          XY(m_cx,m_cy);
          Inc(m_StartLine);
          Paint;
        end else
        begin
          Inc(m_cy);
          XY(m_cx,m_cy);
        end;
      end;

    $4B00: {Left}
      if m_cx<=m_DumpR then
      begin
        Dec(m_cx);
        if ((m_cx-m_DumpL+1) mod m_ItemWidth)=0 then Dec(m_cx);
        if m_cx<m_DumpL then
          if m_StartLine+m_cy>1 then
          begin
            m_cx := m_DumpR;
            goto __up;
          end else
            m_cx := m_DumpL;
        XY(m_cx,m_cy);
      end else
      begin
        Dec(m_cx);
        if m_cx<m_TextL then
        begin
          if m_StartLine+m_cy>1 then
          begin
            m_cx := m_TextR;
            goto __up;
          end else
            m_cx := m_TextL;
        end;
        XY(m_cx,m_cy);
      end;

    $4D00: {Right}
__right:
      if (m_Address<m_bc) and ((m_Address<m_bc-1) or (m_AddressSHL<>0)) then
      begin
        if m_cx<=m_DumpR then
        begin
          Inc(m_cx);
          if ((m_cx-m_DumpL+1) mod m_ItemWidth)=0 then Inc(m_cx);
          if m_cx>m_DumpR then
          begin
            m_cx := m_DumpL;
            goto __down;
          end;
          XY(m_cx,m_cy);
        end else
        begin
          Inc(m_cx);
          if m_cx>m_TextR then
          begin
            m_cx := m_TextL;
            goto __down;
          end;
          XY(m_cx,m_cy);
        end;
      end;

    $4900: {PgUp}
      if m_StartLine<>0 then
      begin
        if m_StartLine>=m_DumpB then
          m_StartLine := m_StartLine - m_DumpB
        else
          m_StartLine := 0;
        Paint;
      end else
        XY(m_cx,1);

    $5100: {PgDn}
      {if Address<bc-LineItems}
      if m_StartLine<MaxStartLine then
      begin
        m_StartLine := m_StartLine + m_DumpB;
        if m_StartLine>MaxStartLine then
          m_StartLine := MaxStartLine;
        if (m_StartLine=MaxStartLine) and (m_cy=m_DumpB) then
          if (m_bc mod m_LineItems<>0) and
             ((m_bc mod m_LineItems)<=(m_Address mod m_LineItems)) then
            XY(m_cx,m_DumpB-1);
        Paint;
      end else
      begin
        if (m_bc mod m_LineItems<>0) and
           ((m_bc mod m_LineItems)<=(m_Address mod m_LineItems)) then
          XY(m_cx,m_DumpB-1)
        else
          XY(m_cx,m_DumpB);
      end;

    $7700, {C+Home}
    $8400: {C+PgUp}
      if (m_StartLine<>0) or (m_cx<>m_DumpL) or (m_cy<>1) then
      begin
        m_StartLine := 0;
        if m_cx<=m_DumpR then
          XY(m_DumpL,1)
        else
          XY(m_TextL,1);
        Paint;
      end;

    $7500, {C+End}
    $7600: {C+PgDn}
      if (m_Address<>m_bc-1) or (m_AddressSHL<>0) then
      begin
        m_StartLine := MaxStartLine;
        if m_cx<=m_DumpR then
          XY(m_DumpR-m_ItemWidth*(m_LineItems-1-((m_bc-1) mod m_LineItems)),m_DumpB)
        else
          XY(m_TextR-1*(m_LineItems-1-((m_bc-1) mod m_LineItems)),m_DumpB);
        Paint;
      end;

    $4700: {Home}
      if m_cx<=m_DumpR then
      begin
        XY(m_DumpL,m_cy)
      end else
      begin
        XY(m_TextL,m_cy);
      end;

    $4F00: {End}
      if m_cx<=m_DumpR then
      begin
        if m_StartLine+m_cy=TotalLines then
          XY(m_DumpR-m_ItemWidth*(m_LineItems-1-((m_bc-1) mod m_LineItems)),m_cy)
        else
          XY(m_DumpR,m_cy)
      end else
      begin
        if m_StartLine+m_cy=TotalLines then
          XY(m_TextR-1*(m_LineItems-1-((m_bc-1) mod m_LineItems)),m_cy)
        else
          XY(m_TextR,m_cy);
      end;

    else if Lo(key)<>0 then
      begin
        if m_cx<=m_DumpR then
        begin
          bKey := unhex(Char(Lo(key)));
          {FastWrite(HexW(bKey),1,1,15);}
          if (bKey<>-1) then
          begin
            w := GetWord;
            w := w and not ($F shl m_AddressSHL);
            w := w or (bKey shl m_AddressSHL);
            if w<=m_wMax then
            begin
              SetWord( w );
              Execute := true;
              PaintLine(m_cy);
              goto __right;
            end;
          end;
        end else
        begin
          w := GetWord;
          w := w and not $FF ;
          w := w or Lo(Key);
          if w<=m_wMax then
          begin
            SetWord( w );
            Execute := true;
            PaintLine(m_cy);
            goto __right;
          end;
        end;
      end;

    end; {case}
  end; {while}
end;


end.