unit uProg;

interface

Uses uCom, uDev;

type
  TProgrammer = object
  private
    m_progress  : TProgress;
    m_Com       : PCom;
    procedure       SetPort( wComNo:word );
    procedure       SetDevice( const sName:string );
  public
    m_Device    : PDevice;

    constructor     Init( progress:TProgress );
    destructor      Free;
    function        Execute( const Cmd,Arg:string ) : boolean;
    function        Port : word;
  end;


implementation

uses TpString;

constructor     TProgrammer.Init( progress:TProgress );
begin
  m_progress:=progress;
  m_Device := nil;
  m_Com := nil;
end;

destructor      TProgrammer.Free;
begin
  if m_Device<>nil then Dispose( m_Device, Free );
  if m_Com<>nil then Dispose( m_Com, Free );
end;


procedure       TProgrammer.SetPort( wComNo:word );
var
  sDeviceName : string;
begin
  sDeviceName := '';
  if m_Device<>nil then
  begin
    sDeviceName := m_Device^.m_sName;
    Dispose( m_Device, Free );
  end;
  if m_Com<>nil then
  begin
    Dispose( m_Com, Free );
  end;
  New( m_Com, Init(wComNo) );
  if sDeviceName<>'' then
  begin
    New( m_Device, Init(sDeviceName,m_Com,m_progress) );
  end;
end;

procedure       TProgrammer.SetDevice( const sName:string );
begin
  if m_Com<>nil then
  begin
    if m_Device<>nil then Dispose( m_Device, Free );
    New( m_Device, Init(sName,m_Com,m_progress) );
  end;
end;


function        TProgrammer.Port : word;
begin
  Port := m_Com^.ComNo;
end;



function        TProgrammer.Execute( const Cmd,Arg:string ) : boolean;
begin

  Execute := false;

  if Equal=CompUCString(Cmd,'DEVICE') then begin

    SetDevice(Arg);
    Execute := true;

  end else if Equal=CompUCString(Cmd,'PORT') then begin

    SetPort( Ord(Arg[4])-$30 );
    Execute := true;

  end else if Equal=CompUCString(Cmd,'READ') then  begin

    Execute := m_Device^.ReadAll;

  end else if Equal=CompUCString(Cmd,'PROGRAM') then begin

    Execute := m_Device^.WriteAll;

  end else if Equal=CompUCString(Cmd,'VERIFY') then begin

    Execute := m_Device^.VerifyAll;

  end else if Equal=CompUCString(Cmd,'ERASE') then begin

    Execute := m_Device^.EraseAll;

  end else if Equal=CompUCString(Cmd,'BLANKCHECK') then begin

    Execute := m_Device^.BlankCheck;

  end else if Equal=CompUCString(Cmd,'LOAD') then  begin

    Execute := m_Device^.LoadFromFile(Arg);

  end else if Equal=CompUCString(Cmd,'SAVE') then  begin

    Execute := m_Device^.SaveToFile(Arg);

  end;
end;



end.